<?php 

function mo_auth_authenticate_user($form, &$form_state, $user) {
  global $base_url;
  if(user_is_logged_in()) {
    drupal_goto('user');
    exit;
  }
  $form_state['uid'] = $user->uid;
  if(isset($_SESSION['mo_auth']['status']) && $_SESSION['mo_auth']['status'] === '1ST_FACTOR_AUTHENTICATED') {
    $user_api_response = $_SESSION['mo_auth']['user'];
    $form['actions'] = array('#type' => 'actions');
    if(!empty($user_api_response->authType)) {
      $form['authType'] = array(
          '#type' => 'hidden',
          '#value' => $user_api_response->authType,
    );
      $authType = AuthenticationType::getAuthType($user_api_response->authType);
      $customer = new MiniorangeCustomerProfile();
      $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $user_api_response->username, 
            $user_api_response->phone, NULL, $user_api_response->authType);
      $form = mo_auth_build_form($form, $base_url, $authType, $miniorange_user);
    } else {
      $form['actions']['submit'] = array(
          '#type' => 'submit',
          '#value' => t('Save'),
          '#attributes' => array('class' => array('element-invisible')),
    );
    }
    return $form;
  }
  drupal_goto('user');
  exit;
}

function mo_auth_authenticate_user_validate($form, &$form_state) {
  
}

function mo_auth_authenticate_user_submit($form, &$form_state) {
  $form_state['rebuild'] = TRUE;
  $uid = $form_state['uid'];
  $user = user_load($uid);
  $user_email = $user->miniorange_registered_email[LANGUAGE_NONE][0]['value'];
  $authType = AuthenticationType::getAuthType($form_state['input']['authType']);
  if($authType['oob'] === FALSE) {
    $token = $form_state['input']['token'];
    $txId = '';
    if(array_key_exists('txId', $form_state['input'])) {
      $txId = $form_state['input']['txId'];
    }
    $customer = new MiniorangeCustomerProfile();
    $miniorange_user = new MiniorangeUser($customer->getCustomerID(), $user_email, NULL, NULL, NULL);
    $authAPIHandler = new AuthenticationAPIHandler($customer->getCustomerID(), $customer->getAPIKey());
    $response = $authAPIHandler->validate($miniorange_user, $txId, $token);
  } else {
    $txId = $form_state['input']['txId'];
    $customer = new MiniorangeCustomerProfile();
    $authAPIHandler = new AuthenticationAPIHandler($customer->getCustomerID(), $customer->getAPIKey());
    $response = $authAPIHandler->getAuthStatus($txId);
  }
  //unset($_SESSION['mo_auth']);
  // read API response
  if($response->status == 'SUCCESS') {
    user_login_submit(array(), $form_state);
    unset($_SESSION['mo_auth']);
  } else if($response->status == 'DENIED') {
    form_set_error('form', t('Authentication denied.'));
    unset($_SESSION['mo_auth']);
    drupal_goto('user');
  } else if($response->status == 'FAILED') {
    $message = t('The passcode you have entered is incorrect.');
    $form = mo_auth_build_form_with_error_message($form_state, $message);
    //var_dump($form_state);
    //exit(0);
    form_set_error('mo-message', $message);
    //drupal_goto('user');
  } else {
    form_set_error('form', t('An error occured while processing your request. Please try again.'));
    unset($_SESSION['mo_auth']);
    drupal_goto('user');
  }
}

function mo_auth_build_form($form, $base_url, $authType, $miniorange_user) {
  $form['header']['#markup'] = t('<div class="mo2f-modal">
              <div class="mo2f-modal-content">
                <div class="mo2f-modal-container mo2f-modal-header">Verify your identity</div>
                <div class="mo2f-modal-container">');
  
  $submit_attributes = array();
  
  $form = mo_auth_build_form_content($form, $base_url, $authType, $miniorange_user);
  $submit_attributes = $form['submit_attributes'];
  unset($form['submit_attributes']);
  $form['loader']['#markup'] = '</div><div class="mo2f-modal-container mo2f-modal-footer">';
  $form['actions']['submit'] = array(
      '#type' => 'submit',
      '#value' => t('Verify'),
      '#attributes' => $submit_attributes,
);
  $form['actions']['cancel'] = array(
      '#markup' => l(t('Back to Login'), 'user'),
      '#suffix' => '</div></div></div>',
);
  return $form;
}

function mo_auth_build_form_content($form, $base_url, $authType, $miniorange_user) {
  
  switch($authType['code']) {
    case AuthenticationType::$EMAIL_VERIFICATION['code']:
      return mo_auth_build_oobemail_form($form, $base_url, $miniorange_user);
    case AuthenticationType::$GOOGLE_AUTHENTICATOR['code']:
      return mo_auth_build_google_authenticator_form($form, $base_url, $miniorange_user);
    case AuthenticationType::$QR_CODE['code']:
      return mo_auth_build_qrcode_authentication_form($form, $base_url, $miniorange_user);
    default:
      return $form;
  }  
}

function mo_auth_build_form_with_error_message(&$form, $error) {
  $form['complete form']['header']['#markup'] = t('<div class="mo2f-modal">
              <div class="mo2f-modal-content">
                <div class="mo2f-modal-container mo2f-modal-header">Verify your identity</div>
                <div class="mo2f-modal-container">');
  $form['complete form']['header']['#markup'] .= t('<div id="mo-message" class="mo2f-message mo2f-message-error">' . $error . '</div>');
  $form['complete form']['header']['#markup'] .= t('<div class="mo2f-info">Enter six digit passcode generated on the Google Authenticator app</div>');
  return $form;
}

function mo_auth_build_google_authenticator_form($form, $base_url, $miniorange_user) {
  $message = t('Please enter the 6 digit passcode generated on your Google Authenticator app.');
  $form['header']['#markup'] .= t('<div class="mo2f-message mo2f-message-status">' . $message . '</div>');
  $form['header']['#markup'] .= t('<div class="mo2f-info">Enter the passcode:</div>');
  $form['token'] = array(
      '#type' => 'textfield',
      '#attributes' => array('placeholder' => t('Enter 6 digit passcode.'), 'class' => array('mo2f-textbox', 'mo2f-textbox-otp'), 'autofocus' => 'true'),
);
  $submit_attributes = array();
  $form['submit_attributes'] = $submit_attributes;
  return $form;
}

function mo_auth_build_qrcode_authentication_form($form, $base_url, $miniorange_user) {
  $response = $_SESSION['mo_auth']['mo_challenge_response'];
  $user_email = $miniorange_user->getEmail();
  $message = t('Please scan the below QR Code from miniOrange Authenticator app.');
  $form['#attached']['js'][] = array(
      'data' => array(
          'mo_authentication' => array(
              'txId' => $response->txId,
              'url' => Miniorange2FAConstants::$AUTH_STATUS_API,
        ),
    ),
      'type' => 'setting',
);
  
  $form['header']['#markup'] .= t('<div class="mo2f-message mo2f-message-status">' . $message . '</div>');
  
  $form['header']['#markup'] .= t('<div class="mo2f-info mo2f-text-center">Please scan the below QR Code with miniOrange Authenticator app to authenticate yourself.</div>');
  $form['header']['#markup'] .= '<div class="mo2f-text-center"><img src="data:image/jpg;base64,' . $response->qrCode . '"></div>';
  
  $form['txId'] = array(
      '#type' => 'hidden',
      '#value' => $response->txId,
);
  $submit_attributes = array('class' => array('element-invisible'));
  $form['submit_attributes'] = $submit_attributes;
  return $form;
}

function mo_auth_build_oobemail_form($form, $base_url, MiniorangeUser $miniorange_user) {
  $response = $_SESSION['mo_auth']['mo_challenge_response'];
  $user_email = $miniorange_user->getEmail();
  $hidden_email = Utilities::getHiddenEmail($user_email);
  $message = t('A verification email is sent to <b>' . $hidden_email . '</b>. Please click on accept link to verify your email.');
  $form['#attached']['js'][] = array(
      'data' => array(
          'mo_authentication' => array(
              'txId' => $response->txId,
              'url' => Miniorange2FAConstants::$AUTH_STATUS_API,
        ),
    ),
      'type' => 'setting',
);
  
  $form['header']['#markup'] .= t('<div class="mo2f-message mo2f-message-status">' . $message . '</div>');
  
  $form['header']['#markup'] .= t('<div class="mo2f-info mo2f-text-center">A verification email is sent to your registered email.</div>
                  <div class="mo2f-info mo2f-text-center">We are waiting for your approval...</div>');
  $image_path = file_create_url($base_url . '/' . drupal_get_path('module', 'mo_auth') . '/includes/images/ajax-loader-login.gif');
  $form['header']['#markup'] .= '<div class="mo2f-text-center"><img src="' . $image_path . '"></div>';
  
  $form['txId'] = array(
      '#type' => 'hidden',
      '#value' => $response->txId,
);
  $submit_attributes = array('class' => array('element-invisible'));
  $form['submit_attributes'] = $submit_attributes;
  return $form;
}
