<?php
class AuthenticationAPIHandler {
  
  private $customerId;
  private $apiKey;
  
  public function __construct($customerId, $apiKey) {
    $this->apiKey = $apiKey;
    $this->customerId = $customerId;
  }

  public function challenge(MiniorangeUser $user) {
    $fields = array(
        'customerKey' => $user->getCustomerId(),
        'username' => $user->getEmail(),
        'authType' => $user->getAuthType(),
        'transactionName' => Miniorange2FAConstants::$PLUGIN_NAME,
    );
    $json = json_encode($fields);
    
    $url = Miniorange2FAConstants::$AUTH_CHALLENGE_API;
    
    return Utilities::callService($this->customerId, $this->apiKey, $url, $json);
  }
  
  public function validate(MiniorangeUser $user, $txId, $token, $answers = NULL) {
    $fields = array(
        'customerKey' => $user->getCustomerId(),
        'username' => $user->getEmail(),
        'txId' => $txId,
        'token' => $token,
        'authType' => $user->getAuthType(),
        'answers' => $answers,
    );
    $json = json_encode($fields);
  
    $url = Miniorange2FAConstants::$AUTH_VALIDATE_API;
  
    return Utilities::callService($this->customerId, $this->apiKey, $url, $json);
  }
  
  public function getAuthStatus($txId) {
    $fields = array(
      'txId' => $txId,
    );
    $json = json_encode($fields);
    
    $url = Miniorange2FAConstants::$AUTH_STATUS_API;
    
    return Utilities::callService($this->customerId, $this->apiKey, $url, $json);
  }
  
  public function getGoogleAuthSecret(MiniorangeUser $user) {
    $fields = array(
        'customerKey' => $user->getCustomerId(),
        'username' => $user->getEmail(),
    );
    $json = json_encode($fields);
  
    $url = Miniorange2FAConstants::$AUTH_GET_GOOGLE_AUTH_API;
  
    return Utilities::callService($this->customerId, $this->apiKey, $url, $json);
  }
  
  public function registerGoogleAuthenticator(MiniorangeUser $user, $secret, $otpToken) {
    $fields = array(
        'customerKey' => $user->getCustomerId(),
        'username' => $user->getEmail(),
        'secret' => $secret,
        'otpToken' => $otpToken,
    );
    $json = json_encode($fields);
  
    $url = Miniorange2FAConstants::$AUTH_REGISTER_GOOGLE_AUTH_API;
  
    return Utilities::callService($this->customerId, $this->apiKey, $url, $json);
  }
  
  public function registerQRCode(MiniorangeUser $user) {
    $fields = array(
        'username' => $user->getEmail(),
    );
    $json = json_encode($fields);
  
    $url = Miniorange2FAConstants::$AUTH_REGISTER_MOBILE_API;
  
    return Utilities::callService($this->customerId, $this->apiKey, $url, $json);
  }
  
  public function getRegistrationStatus($txId) {
    $fields = array(
        'txId' => $txId,
    );
    $json = json_encode($fields);
  
    $url = Miniorange2FAConstants::$AUTH_REGISTRATION_STATUS_API;
  
    return Utilities::callService($this->customerId, $this->apiKey, $url, $json);
  }
}